classdef environment < handle
    %ENVIRONMENT Summary of this class goes here
    %   Detailed explanation goes here
    
    properties (Constant = true, GetAccess = private)
        gamma = 1.402;      % Ratio of specific heats for air [-]
        R = 8.3144598;      % Gas constant [J/K/mol]
        M = 28.97e-3;       % Molecular mass of air [kg/mol]
        T0C = 273.15;       % Temperature for zero Celsius [K]  
    end
    properties (SetAccess = immutable)
        p0                  % Atmospheric pressure [Pa]
        T0                  % Atmospheric temperature [K]
        h_rel               % Relative humidity [%]
    end
    
    methods
        function obj = environment(p0, T0_C, h_rel)
            % ENVIRONMENT Constructor
            % OBJ = Environment(P0, T0_C, HREL) Creates a new instance of 
            % the Environment class. P0 is the  ambient pressure [Pa], T0_C 
            % is the ambient temperature [Celsius degrees], and HREL is the 
            % relative humidity [%]. Each input parameter can be empty, 
            % then, the following default values are used: P0 = 101325 Pa, 
            % T0_C = 20 C, HREL = 0%.
            %
            % Examples:
            %   Env = Environment(100200, 23, 40);
            %   Creates an environment with P0 = 100200 Pa, T0 = 23 Celsius
            %   and HREL = 40%.
            
            % Check relative humidity
            if nargin < 3 || isempty(h_rel)
                h_rel = 0;
            else
                if (h_rel < 0 || h_rel > 100)
                    error('Environment:Environment:badHRel', ...
                        'Relative humidity must be in between 0 %% and 100 %%.');
                end
            end
            
            % Check ambient temperature
            if nargin < 2 || isempty(T0_C)
                T0_C = 20;
            else
                if (T0_C <= -environment.T0C)
                    error('Environment:badT0',...
                        'Ambient temperature must be greater than 0 K.');
                end
            end
            
            % Check ambient pressure
            if nargin < 1 || isempty(p0)
                p0 = 101325;
                if (p0 <= 0)
                    error('Environment:badP0',...
                        'Ambient pressure must be greater than 0 Pa.');
                end
            end
            
            % Store te values
            obj.p0 = p0;
            obj.T0 = T0_C + environment.T0C;
            obj.h_rel = h_rel;
        end % of Constructor
        
        function T = getTemperatureInCelsius(obj)
            T = obj.T0 - obj.T0C;
        end
        
        function rho = get_density(obj)
            % GETDENSITY Calculate equilibrium density
            % RHO = Environment.getDensity() Returns the equilibrium
            % density of air in [kg/m^3].
            rho = 1.2929*(1-0.0037*(obj.T0 - obj.T0C));
        end

        function c = get_speedofsound(obj)
            % C = Environment.get_speedofsound() Returns the speed of sound 
            % in [m/s] units using the relation $c = \sqrt{\gamma R T_0}$
            c = sqrt(environment.gamma * environment.R / environment.M * obj.T0);
        end
       function Z = get_impedance(obj)
            % GETDENSITY Calculate equilibrium density
            % RHO = Environment.getDensity() Returns the equilibrium
            % density of air in [kg/m^3].
            Z = obj.get_density.*obj.get_speedofsound;
        end
        
        function str = getDescriptionString(obj)
            str = sprintf('p0: %.0f Pa, T: %.1f C, hrel: %.1f %%', ...
                obj.p0, obj.T0 - obj.T0C, obj.h_rel);
        end

    end
end

