classdef multilayered_absorber < handle
    %MULTILAYERED_ABSORBER
    % Contains multiple layers of absorbent material,
    % Materials are indexed from top (closest to air)
    
    properties (SetAccess = protected)
        layers
        total_transfer_matrix
        total_scattering_matrix
        boundary_p_vals % cell of
    end
    
    properties (SetAccess = immutable)
        env
    end
    
    methods
        function obj = multilayered_absorber(env)
            obj.env = env;
            obj.layers = {};
        end
        
        function add_layer(obj,varargin)
            %Add new layer to the absorber
            % Input arguments:
            %   layer_in : layer to add
            %   layer_ix : optional, position of layer to add
            %               if not present, layer is added to the bottom
            %               of the absorber
            %            type, D, FR, model
            N = length(obj.layers);
            switch varargin{1}
                case 'air'
                    obj.layers{N+1} = air_layer(N+1,'air',varargin{2});
                case 'limp_membrane'
                    obj.layers{N+1} = limp_membrane_layer(N+1,'wood',varargin{2});
                case 'porous'
                    obj.layers{N+1} = porous_layer(N+1,'rock_wool',varargin{2},varargin{3},varargin{4});
                case 'perforated_panel'
                    obj.layers{N+1} = perforated_panel(N+1,'perforated_panel',varargin{2},varargin{3},varargin{4});
                case 'slotted_panel'
                    obj.layers{N+1} = perforated_panel(N+1,'perforated_panel',varargin{2},varargin{3},varargin{4});
                case 'rigid_termination'
                    obj.layers{N+1} = rigid_termination(N+1,'rigid_termination');
                case 'halfspace'
                    obj.layers{N+1} = halfspace(N+1,'halfspace');
            end
            
        end
        
        function Zsurf = calculate_surface_impedance(obj, pw_dir, f)
            % Calculation based on transfer matrices
            N = length(obj.layers);
            T_total = repmat(eye(2),[1,1,length(f)]);
            for n = 1:N
                obj.layers{n}.calculate_acoustic_properties(obj.env,f);
                obj.layers{n}.calculate_transfer_matrix(pw_dir);
                if ~(strcmp(obj.layers{n}.material, 'rigid_termination')||strcmp(obj.layers{n}.material, 'halfspace'))
                    T = obj.layers{n}.get_transfer_matrix;
                    T_total = MatrixMult(T_total,T);
                else
                    Zspec = obj.layers{n}.get_impedance;
                    Zsurf = (squeeze(T_total(1,1,:)) + squeeze(T_total(1,2,:))./Zspec)./...
                        (squeeze(T_total(2,1,:)) + squeeze(T_total(2,2,:))./Zspec);
                    break;
                end
            end
            obj.total_transfer_matrix = T_total;
        end
        
        function obj = calculate_boundary_pressures(obj,f,pw_dir, p_in)
            R_total = calculate_reflection_coefficient(obj, pw_dir, f);
            P_surface = p_in.*(1 + R_total);
            V_surface = P_surface.*cos(pw_dir)./(obj.env.get_speedofsound.*obj.env.get_density);
            N = length(obj.layers);
            
            T_total = repmat(eye(2),[1,1,length(f)]);
            p_bottom = [P_surface.';V_surface.'];
            obj.boundary_p_vals = {};
            obj.boundary_p_vals{1} = struct('p_top',[],'p_bottom',p_bottom,'p_in',p_in,'p_out',R_total.*p_in);
            for n = 1:N-1
                T = obj.layers{n}.get_transfer_matrix;
                T_total = MatrixMult(T_total,T);
                p_top = obj.boundary_p_vals{n}.p_bottom;
                p_bottom = MatrixVecMult( T, p_top, 'inv' );
                obj.boundary_p_vals{n+1} = struct('p_top',p_top,'p_bottom',p_bottom);
            end
        end
        
        function simulate_field(obj, Pin, freq, pw_dir,dx,L)
            N = length(obj.layers);
            total_p = [];
            t = (0:length(freq)-1)'/freq(end)-L/obj.env.get_speedofsound;
            y = (-L/4:dx:L/4);
            for n = 1:N
                if n == 1
                    x = (-L:dx:0);
                    [X,Y] = meshgrid(x,y);
                    P = zeros(numel(X),length(t));
                    c0 = obj.env.get_speedofsound;
                    for f = 1 : length(freq)
                        f
                        w = 2*pi*freq(f);
                        Et = exp(1i*w*t.');
                        pin  = obj.boundary_p_vals{n}.p_in(f);
                        pout = obj.boundary_p_vals{n}.p_out(f);
                        pin(isnan(pin)) = 1;
                        pout(isnan(pout)) = 1;
                        P = P + Pin(f) *( bsxfun( @times, pin.* ( exp(-1i.*w/c0.*X(:) )  ) , Et  ) +...
                            bsxfun( @times, pout.*( exp( 1i.*w/c0.*X(:) )  ) , Et  ) );
                    end
                    P = reshape(P,size(X,1),size(X,2),length(t));
                    p_total = [P];
                else
                    D = obj.layers{n-1}.thickness;
                    x = (0:dx:D);
                    [X,Y] = meshgrid(x,y);
                    P = zeros(numel(X),length(t));
                    kx = cos(pw_dir)*obj.layers{n-1}.wavenumber;
                    for f = 2 : length(freq)
                        f
%                        P0 = obj.boundary_p_vals{n}.p_top(1,:).';
%                        Pd = obj.boundary_p_vals{n}.p_bottom(1,:).';
%                        P0(isnan(P0)) = 0;
%                        Pd(isnan(Pd)) = 0;
                        M = [1 1;kx(f)./(2*pi*freq(f)*obj.env.get_density) -kx(f)./(2*pi*freq(f)*obj.env.get_density)];
                        Pp = pinv(M)*obj.boundary_p_vals{n}.p_top(:,f);
                        Pp(isnan(Pp)) = 0;
                        w = 2*pi*freq(f);
                        Et = exp(1i*w*t.');
                         P = P + Pin(f) *( bsxfun( @times, Pp(1).* ( exp(-1i.*w/c0.*X(:) )  ) , Et  ) +...
                            bsxfun( @times, Pp(2).*( exp( 1i.*w/c0.*X(:) )  ) , Et  ) );

%                        P  = P + bsxfun(@times, ( P0(f).*sin( kx(f)*(X(:)-D) ) + Pd(f).*sin(kx(f).*X(:)) )./sin(kx(f)*D), Et);
                    end
                    P = reshape(P,size(X,1),size(X,2),length(t));
                    p_total = [p_total P];
                end
            end
            
            %%
            figure;
            p1 = pcolor(real(squeeze(p_total(:,:,1))));
            shading interp
            axis equal tight
            for ti = 1 : length(t)
                set(p1,'CData',abs(squeeze(p_total(:,:,ti))));
                pause(0.1);
                caxis([-1,1]*1e2)
            end
            %%
        end
        
        
        function R = calculate_reflection_coefficient(obj, pw_dir, f)
            Zsurf = obj.calculate_surface_impedance(pw_dir, f);
            Z0 = obj.env.get_impedance;
            R = (Zsurf.*cos(pw_dir)-Z0)./(Zsurf.*cos(pw_dir) + Z0);
        end
        
        function alpha = calculate_absorption_coefficient(obj, pw_dir, f)
            R = obj.calculate_reflection_coefficient(pw_dir, f);
            alpha = 1 - abs(R).^2;
        end
        
        function Zsurf = calculate_surface_impedance_2(obj, pw_dir, f)
            % Calculation with direct impedance formulation
            N = length(obj.layers);
            for n = N : -1 : 1
                obj.layers{n}.calculate_acoustic_properties(obj.env,f);
                if (strcmp(obj.layers{n}.material, 'rigid_termination')||strcmp(obj.layers{n}.material, 'halfspace'))
                    Zsurf = obj.layers{n}.get_impedance;
                else
                    Zspec = obj.layers{n}.get_impedance;
                    k = obj.layers{n}.get_wavenumber;
                    kx = k.*cos(pw_dir);
                    d = obj.layers{n}.thickness;
                    Zsurf = (-1i*Zspec.*cot(kx.*d) + Zspec.^2./Zsurf)./(1-1i*Zspec.*cot(kx.*d)./Zsurf);
                end
            end
        end
        
    end
end

